#!/usr/bin/env python3
import os.path
from time import sleep
from typing import Optional
from argparse import ArgumentParser, ArgumentError

from home.config import config
from home.mqtt import MqttNode, MqttWrapper, get_mqtt_modules

mqtt_node: Optional[MqttNode] = None
mqtt: Optional[MqttWrapper] = None


if __name__ == '__main__':
    parser = ArgumentParser()
    parser.add_argument('--node-id', type=str, required=True)
    parser.add_argument('--modules', type=str, choices=get_mqtt_modules(), nargs='*',
                        help='mqtt modules to include')
    parser.add_argument('--switch-relay', choices=[0, 1], type=int,
                        help='send relay state')
    parser.add_argument('--push-ota', type=str, metavar='OTA_FILENAME',
                        help='push ota, argument receives filename')
    parser.add_argument('--node-secret', type=str,
                        help='node admin password')

    config.load_app('mqtt_util', parser=parser)
    arg = parser.parse_args()

    if (arg.switch_relay is not None or arg.node_secret is not None) and 'relay' not in arg.modules:
        raise ArgumentError(None, '--relay is only allowed when \'relay\' module included in --modules')

    mqtt = MqttWrapper(randomize_client_id=True)
    mqtt_node = MqttNode(node_id=arg.node_id, node_secret=arg.node_secret)

    mqtt.add_node(mqtt_node)

    # must-have modules
    ota_module = mqtt_node.load_module('ota')
    mqtt_node.load_module('diagnostics')

    if arg.modules:
        for m in arg.modules:
            module_instance = mqtt_node.load_module(m)
            if m == 'relay' and arg.switch_relay is not None:
                if not arg.node_secret:
                    raise ArgumentError(None, '--switch-relay requires --node-secret')
                module_instance.switchpower(arg.switch_relay == 1)

    mqtt.configure_tls()
    try:
        mqtt.connect_and_loop(loop_forever=False)

        if arg.push_ota:
            if not os.path.exists(arg.push_ota):
                raise OSError(f'--push-ota: file \"{arg.push_ota}\" does not exists')
            if not arg.node_secret:
                raise ArgumentError(None, 'pushing OTA requires --node-secret')

            ota_module.push_ota(arg.push_ota, 1)

        while True:
            sleep(0.1)

    except KeyboardInterrupt:
        mqtt.disconnect()
